/**
 * \mainpage CSC3502 : PonyTracker
 *
 * Les fonctionnalités de cette application sont les suivantes :
 * <ul>
 * <li> Affichage d'une mélodie via une interface dynamique</li>
 * <li> Edition de la mélodie</li>
 * <li> Lecture de la mélodie</li>
 * </ul>
 *
 * @file main.c
 *
 * Programme principal du Ponytracker
 *
 * @version 1.1
 * @author Module CSC3502 : Projet Informatique 1ère année TELECOM SudParis
 * @date  Mai 2014
*/

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

#include "motifs/motifs.h"
#include "melodie/melodie.h"
#include "samples/samples.h"
#include "instruments/instruments.h"
#include "lecture/lecture.h"
//#include <SDL2/SDL.h>
//#include <SDL2/SDL_mixer.h>
//#include <SDL2/SDL_main.h>

#define DEF_NBR_TMP 16
#define MAX_PATTERNS 256
#define NBR_INST 10
#define NBR_SMPL 5

// Variables globales
Motif* melodie[MAX_PATTERNS] = {NULL};
Motif* liste = NULL;
Instrument* inst[NBR_INST] = {NULL};
Sample* smpl[NBR_SMPL] = {NULL};
int nbrPortees = 1;
int taille;

GtkBuilder *monBuilder;
char *Do="Do";
char *DoD="Do#";
char *Re="Re";
char *ReD="Re#";
char *Mi="Mi";
char *Fa="Fa";
char *FaD="Fa#";
char *Sol="Sol";
char *SolD="Sol#";
char *La="La";
char *LaD="La#";
char *Si="Si";
char *N1="1";
char *N2="2";
char *N3="3";
char *N4="4";
char *N5="5";


// Fonctions de débug

void afficherMotif(Motif* m, int nbrPortees){
	printf("%s  :\n",m->nom);
	for (int tmp = 0; tmp < m->nbrTmp; tmp++){
		for (int portee = 0; portee < nbrPortees; portee++){
			printf("%d ",getNote(m,portee,tmp));
		}
		printf("\n");
	}
}

// Fin des fonctions de débug

void debut(Motif* melodie[], Instrument* inst[], Sample* smpl[], Motif* liste, int nbrPortees){
	ajouterMotif(&liste, DEF_NBR_TMP, nbrPortees);
	Motif* m = liste;

	int i;

	// En guise d'exemple, voici la mélodie du générique de My Little Pony : Friendship is Magic.

	// My Little Pony.
	
	for (i = 0; i < DEF_NBR_TMP; i++) {
		definirNote(m,0,i,-1);
		definirOctave(m,0,i,3);
	}

	definirNote(m,0,0,1);
	definirInstrument(m,0,0,0);
	definirInstrument(m,0,2,-1);
	definirNote(m,0,4,0);
	definirInstrument(m,0,4,0);
	definirNote(m,0,5,1);
	definirInstrument(m,0,5,0);
	definirInstrument(m,0,6,-1);
	definirNote(m,0,7,3);
	definirInstrument(m,0,7,0);
	definirInstrument(m,0,8,-1);
	definirNote(m,0,9,1);
	definirInstrument(m,0,9,0);
	definirInstrument(m,0,12,-1);
	
	ajouterMotif(&liste, DEF_NBR_TMP, nbrPortees);
	Motif* n = liste;

	// I used to wonder what friendship could be.
	
	for (i = 0; i < DEF_NBR_TMP; i++) {
		definirNote(n,0,i,-1);
		definirOctave(n,0,i,3);
	}

	definirNote(n,0,1,5);
	definirInstrument(n,0,1,0);
	definirNote(n,0,2,5);
	definirInstrument(n,0,2,0);
	definirNote(n,0,3,5);
	definirInstrument(n,0,3,0);
	definirNote(n,0,4,5);
	definirInstrument(n,0,4,0);
	definirInstrument(n,0,5,-1);
	definirNote(n,0,6,1);
	definirInstrument(n,0,6,0);
	definirNote(n,0,7,3);
	definirInstrument(n,0,7,0);
	definirInstrument(n,0,8,-1);
	definirNote(n,0,9,1);
	definirInstrument(n,0,9,0);
	definirInstrument(n,0,10,-1);
	definirNote(n,0,11,5);
	definirInstrument(n,0,11,0);
	definirInstrument(n,0,12,-1);
	definirNote(n,0,13,3);
	definirInstrument(n,0,13,0);
	definirInstrument(n,0,14,-1);
	definirNote(n,0,15,1);
	definirInstrument(n,0,15,0);

	ajouterMotif(&liste, DEF_NBR_TMP, nbrPortees);
	Motif* p = liste;

	// Until you all shared its magic with me.
	
	for (i = 0; i < DEF_NBR_TMP; i++) {
		definirNote(p,0,i,-1);
		definirOctave(p,0,i,3);
	}

	definirNote(p,0,0,5);
	definirInstrument(p,0,0,0);
	definirNote(p,0,1,8);
	definirInstrument(p,0,1,0);
	definirInstrument(p,0,2,-1);
	definirNote(p,0,3,8);
	definirInstrument(p,0,3,0);
	definirInstrument(p,0,4,-1);
	definirNote(p,0,5,5);
	definirInstrument(p,0,5,0);
	definirInstrument(p,0,6,-1);
	definirNote(p,0,7,3);
	definirInstrument(p,0,7,0);
	definirInstrument(p,0,8,-1);
	definirNote(p,0,9,1);
	definirInstrument(p,0,9,0);
	definirNote(p,0,10,5);
	definirInstrument(p,0,10,0);
	definirInstrument(p,0,11,-1);
	definirNote(p,0,12,3);
	definirInstrument(p,0,12,0);
	definirInstrument(p,0,14,-1);

	ajouterMotifVirtuel(melodie,0,m);
	ajouterMotifVirtuel(melodie,1,n);
	ajouterMotifVirtuel(melodie,2,m);
	ajouterMotifVirtuel(melodie,3,p);

	// Définir les instruments les samples

	ajouterSample(smpl,0,"lecture/0.wav");
	setHauteur(smpl[0],440.0);

	ajouterInstrument(inst,0);
	setSample(inst[0],smpl[0]);
}

int main( int argc, char *argv[] ){
  debut(melodie, inst, smpl, liste, nbrPortees);
  Motif* m = melodie[0];
  GtkWidget *maFenetre;
  char str[20];
  gtk_init( &argc, &argv );
  
  monBuilder = gtk_builder_new();
  
  gtk_builder_add_from_file( monBuilder, "GUI/ponytrackertest.glade", NULL);
  gtk_builder_connect_signals( monBuilder, NULL);
  
  maFenetre = GTK_WIDGET( gtk_builder_get_object( monBuilder, "fenetreprincipale"));
  
GtkButton *button0_0 = (GtkButton*) gtk_builder_get_object(monBuilder, "button16");
	switch (getNote(m,0,0)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,0,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_0, str);

	GtkButton *button0_1 = (GtkButton*) gtk_builder_get_object(monBuilder, "button18");
	switch (getNote(m,0,1)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,0,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_1, str);

	GtkButton *button0_2 = (GtkButton*) gtk_builder_get_object(monBuilder, "button20");
	switch (getNote(m,0,2)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,0,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_2, str);

	GtkButton *button0_3 = (GtkButton*) gtk_builder_get_object(monBuilder, "button21");
	switch (getNote(m,0,3)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,0,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_3, str);

	GtkButton *button0_4 = (GtkButton*) gtk_builder_get_object(monBuilder, "button22");
	switch (getNote(m,0,4)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,0,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_4, str);

	GtkButton *button0_5 = (GtkButton*) gtk_builder_get_object(monBuilder, "button23");
	switch (getNote(m,0,5)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}
	
	switch (getOctave(m,0,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_5, str);

	GtkButton *button0_6 = (GtkButton*) gtk_builder_get_object(monBuilder, "button25");
	switch (getNote(m,0,6)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,0,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_6, str);

	GtkButton *button0_7 = (GtkButton*) gtk_builder_get_object(monBuilder, "button24");
	switch (getNote(m,0,7)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,0,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_7, str);

	GtkButton *button0_8 = (GtkButton*) gtk_builder_get_object(monBuilder, "button26");
	switch (getNote(m,0,8)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,0,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_8, str);

	GtkButton *button0_9 = (GtkButton*) gtk_builder_get_object(monBuilder, "button27");
	switch (getNote(m,0,9)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,0,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_9, str);

	GtkButton *button0_10 = (GtkButton*) gtk_builder_get_object(monBuilder, "button28");
	switch (getNote(m,0,10)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,0,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_10, str);

	GtkButton *button0_11 = (GtkButton*) gtk_builder_get_object(monBuilder, "button29");
	switch (getNote(m,0,11)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,0,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_11, str);

	GtkButton *button0_12 = (GtkButton*) gtk_builder_get_object(monBuilder, "button30");
	switch (getNote(m,0,12)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,0,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_12, str);

	GtkButton *button0_13 = (GtkButton*) gtk_builder_get_object(monBuilder, "button31");
	switch (getNote(m,0,13)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,0,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_13, str);

	GtkButton *button0_14 = (GtkButton*) gtk_builder_get_object(monBuilder, "button32");
	switch (getNote(m,0,14)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,0,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_14, str);

	GtkButton *button0_15 = (GtkButton*) gtk_builder_get_object(monBuilder, "button33");
	switch (getNote(m,0,15)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,0,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,0,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,0,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,0,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button0_15, str);

	GtkButton *button1_0 = (GtkButton*) gtk_builder_get_object(monBuilder, "button17");
	switch (getNote(m,1,0)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,1,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_0, str);

	GtkButton *button1_1 = (GtkButton*) gtk_builder_get_object(monBuilder, "button19");
	switch (getNote(m,1,1)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,1,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_1, str);

	GtkButton *button1_2 = (GtkButton*) gtk_builder_get_object(monBuilder, "button36");
	switch (getNote(m,1,2)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,1,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_2, str);

	GtkButton *button1_3 = (GtkButton*) gtk_builder_get_object(monBuilder, "button37");
	switch (getNote(m,1,3)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,1,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_3, str);

	GtkButton *button1_4 = (GtkButton*) gtk_builder_get_object(monBuilder, "button38");
	switch (getNote(m,1,4)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,1,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_4, str);

	GtkButton *button1_5 = (GtkButton*) gtk_builder_get_object(monBuilder, "button39");
	switch (getNote(m,1,5)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}
	
	switch (getOctave(m,1,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_5, str);

	GtkButton *button1_6 = (GtkButton*) gtk_builder_get_object(monBuilder, "button40");
	switch (getNote(m,1,6)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,1,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_6, str);

	GtkButton *button1_7 = (GtkButton*) gtk_builder_get_object(monBuilder, "button41");
	switch (getNote(m,1,7)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,1,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_7, str);

	GtkButton *button1_8 = (GtkButton*) gtk_builder_get_object(monBuilder, "button42");
	switch (getNote(m,1,8)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,1,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_8, str);

	GtkButton *button1_9 = (GtkButton*) gtk_builder_get_object(monBuilder, "button43");
	switch (getNote(m,1,9)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,1,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_9, str);

	GtkButton *button1_10 = (GtkButton*) gtk_builder_get_object(monBuilder, "button44");
	switch (getNote(m,1,10)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,1,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_10, str);

	GtkButton *button1_11 = (GtkButton*) gtk_builder_get_object(monBuilder, "button45");
	switch (getNote(m,1,11)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,1,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_11, str);

	GtkButton *button1_12 = (GtkButton*) gtk_builder_get_object(monBuilder, "button46");
	switch (getNote(m,1,12)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,1,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_12, str);

	GtkButton *button1_13 = (GtkButton*) gtk_builder_get_object(monBuilder, "button47");
	switch (getNote(m,1,13)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,1,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_13, str);

	GtkButton *button1_14 = (GtkButton*) gtk_builder_get_object(monBuilder, "button48");
	switch (getNote(m,1,14)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,1,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_14, str);

	GtkButton *button1_15 = (GtkButton*) gtk_builder_get_object(monBuilder, "button49");
	switch (getNote(m,1,15)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,1,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,1,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,1,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,1,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button1_15, str);
GtkButton *button2_0 = (GtkButton*) gtk_builder_get_object(monBuilder, "button52");
	switch (getNote(m,2,0)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,2,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,0)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_0, str);

	GtkButton *button2_1 = (GtkButton*) gtk_builder_get_object(monBuilder, "button54");
	switch (getNote(m,2,1)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,2,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,1)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_1, str);

	GtkButton *button2_2 = (GtkButton*) gtk_builder_get_object(monBuilder, "button56");
	switch (getNote(m,2,2)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,2,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,2)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_2, str);

	GtkButton *button2_3 = (GtkButton*) gtk_builder_get_object(monBuilder, "button58");
	switch (getNote(m,2,3)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,2,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,3)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_3, str);

	GtkButton *button2_4 = (GtkButton*) gtk_builder_get_object(monBuilder, "button60");
	switch (getNote(m,2,4)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,2,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,4)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_4, str);

	GtkButton *button2_5 = (GtkButton*) gtk_builder_get_object(monBuilder, "button62");
	switch (getNote(m,2,5)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}
	
	switch (getOctave(m,2,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,5)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_5, str);

	GtkButton *button2_6 = (GtkButton*) gtk_builder_get_object(monBuilder, "button64");
	switch (getNote(m,2,6)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,2,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,6)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_6, str);

	GtkButton *button2_7 = (GtkButton*) gtk_builder_get_object(monBuilder, "button66");
	switch (getNote(m,2,7)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,2,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,7)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_7, str);

	GtkButton *button2_8 = (GtkButton*) gtk_builder_get_object(monBuilder, "button68");
	switch (getNote(m,2,8)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,2,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,8)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_8, str);

	GtkButton *button2_9 = (GtkButton*) gtk_builder_get_object(monBuilder, "button69");
	switch (getNote(m,2,9)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,2,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,9)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_9, str);

	GtkButton *button2_10 = (GtkButton*) gtk_builder_get_object(monBuilder, "button70");
	switch (getNote(m,2,10)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,2,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,10)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_10, str);

	GtkButton *button2_11 = (GtkButton*) gtk_builder_get_object(monBuilder, "button71");
	switch (getNote(m,2,11)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,2,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,11)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_11, str);

	GtkButton *button2_12 = (GtkButton*) gtk_builder_get_object(monBuilder, "button72");
	switch (getNote(m,2,12)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	
	switch (getOctave(m,2,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,12)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_12, str);

	GtkButton *button2_13 = (GtkButton*) gtk_builder_get_object(monBuilder, "button73");
	switch (getNote(m,2,13)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,2,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,13)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_13, str);

	GtkButton *button2_14 = (GtkButton*) gtk_builder_get_object(monBuilder, "button74");
	switch (getNote(m,2,14)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}

	
	switch (getOctave(m,2,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,14)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_14, str);

	GtkButton *button2_15 = (GtkButton*) gtk_builder_get_object(monBuilder, "button75");
	switch (getNote(m,2,15)) {
	case 0 :
	  strcpy(str,"Do ");
	  break;
	case 1 :
	  strcpy(str,"Do# ");
	  break;
	case 2 :
	  strcpy(str,"Re ");
	  break;
	case 3 :
	  strcpy(str,"Re# ");
	  break;
	case 4 :
	  strcpy(str,"Mi ");
	  break;
	case 5 :
	  strcpy(str,"Fa ");
	  break;
	case 6 :
	  strcpy(str,"Fa# ");
	  break;
	case 7 :
	  strcpy(str,"Sol ");
	  break;
	case 8 :
	  strcpy(str,"Sol# ");
	  break;
	case 9 : 
	  strcpy(str,"La ");
	  break;
	case 10 :
	  strcpy(str,"La# ");
	  break;
	case 11 :
	  strcpy(str,"Si ");
	  break;
	default :
	  break;
	}


	
	switch (getOctave(m,2,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getInstrument(m,2,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getVolume(m,2,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	switch (getEffet(m,2,15)) {
	case 0 :
	  strcat(str, "0 ");
	  break;
	case 1 : 
	  strcat(str, "1 ");
	  break;
	case 2 :
	  strcat(str, "2 ");
	  break;
	case 3 :
	  strcat(str, "3 ");
	  break;
	case 4 :
	  strcat(str, "4 ");
	  break;
	default :
	  break;
	}

	gtk_button_set_label(button2_15, str);


	gtk_widget_show_all (maFenetre);
	gtk_main();
	return EXIT_SUCCESS;
}

void fenetreSample ( int argc, char *argv[] )
{

  GtkWidget *maFenetreSample = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutsample"));

  gtk_widget_show_all (maFenetreSample);
}

void fenetreInstrument ( int argc, char *argv[] )
{
  GtkWidget *maFenetreInstrument = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutinstrument"));

  gtk_widget_show_all (maFenetreInstrument);
}

void fenetreMotif ( int argc, char *argv[] )
{
  GtkWidget *maFenetreMotif = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutmotif"));

  gtk_widget_show_all (maFenetreMotif);
}

void fenetreNote0_0 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_0"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_0 (GtkWidget *widget, gpointer *data){

	Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_0");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_0");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_0");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_0");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_0");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,0,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,0,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,0,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,0,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,0,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,0,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,0,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,0,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,0,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,0,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,0,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,0,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,0,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,0,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,0,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,0,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,0,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,0,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,0,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,0,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,0,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,0,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,0,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,0,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,0,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,0,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,0,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,0,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,0,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,0,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,0,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,0,5);
}
void fenetreNote0_1 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_1"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_1 (GtkWidget *widget, gpointer *data){

Motif* m = melodie[0];


  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_1");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_1");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_1");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_1");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_1");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,1,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,1,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,1,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,1,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,1,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,1,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,1,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,1,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,1,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,1,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,1,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,1,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,1,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,1,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,1,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,1,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,1,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,1,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,1,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,1,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,1,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,1,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,1,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,1,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,1,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,1,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,1,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,1,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,1,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,1,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,1,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,1,5);
}
void fenetreNote0_2 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_2"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_2 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_2");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_2");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_2");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_2");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_2");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,2,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,2,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,2,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,2,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,2,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,2,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,2,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,2,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,2,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,2,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,2,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,2,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,2,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,2,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,2,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,2,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,2,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,2,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,2,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,2,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,2,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,2,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,2,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,2,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,2,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,2,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,2,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,2,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,2,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,2,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,2,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,2,5);
}
void fenetreNote0_3 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_3"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_3 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_3");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_3");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_3");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_3");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_3");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,3,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,3,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,3,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,3,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,3,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,3,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,3,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,3,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,3,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,3,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,3,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,3,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,3,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,3,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,3,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,3,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,3,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,3,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,3,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,3,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,3,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,3,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,3,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,3,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,3,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,3,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,3,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,3,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,3,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,3,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,3,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,3,5);
}
void fenetreNote0_4 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_4"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_4 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_4");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_4");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_4");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_4");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_4");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,4,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,4,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,4,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,4,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,4,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,4,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,4,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,4,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,4,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,4,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,4,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,4,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,4,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,4,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,4,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,4,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,4,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,4,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,4,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,4,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,4,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,4,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,4,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,4,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,4,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,4,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,4,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,4,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,4,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,4,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,4,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,4,5);
}
void fenetreNote0_5 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_5"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_5 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_5");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_5");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_5");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_5");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_5");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,5,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,5,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,5,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,5,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,5,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,5,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,5,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,5,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,5,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,5,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,5,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,5,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,5,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,5,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,5,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,5,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,5,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,5,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,5,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,5,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,5,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,5,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,5,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,5,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,5,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,5,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,5,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,5,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,5,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,5,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,5,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,5,5);
}
void fenetreNote0_6 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_6"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_6 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_6");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_6");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_6");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_6");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_6");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,6,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,6,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,6,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,6,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,6,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,6,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,6,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,6,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,6,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,6,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,6,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,6,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,6,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,6,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,6,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,6,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,6,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,6,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,6,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,6,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,6,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,6,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,6,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,6,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,6,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,6,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,6,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,6,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,6,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,6,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,6,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,6,5);
}
void fenetreNote0_7 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_7"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_7 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_7");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_7");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_7");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_7");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_7");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,7,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,7,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,7,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,7,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,7,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,7,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,7,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,7,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,7,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,7,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,7,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,7,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,7,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,7,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,7,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,7,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,7,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,7,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,7,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,7,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,7,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,7,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,7,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,7,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,7,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,7,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,7,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,7,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,7,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,7,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,7,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,7,5);
}
void fenetreNote0_8 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_8"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_8 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_8");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_8");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_8");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_8");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_8");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,8,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,8,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,8,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,8,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,8,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,8,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,8,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,8,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,8,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,8,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,8,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,8,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,8,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,8,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,8,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,8,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,8,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,8,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,8,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,8,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,8,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,8,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,8,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,8,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,8,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,8,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,8,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,8,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,8,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,8,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,8,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,8,5);
}
void fenetreNote0_9 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_9"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_9 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_9");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_9");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_9");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_9");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_9");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,9,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,9,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,9,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,9,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,9,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,9,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,9,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,9,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,9,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,9,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,9,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,9,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,9,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,9,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,9,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,9,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,9,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,9,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,9,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,9,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,9,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,9,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,9,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,9,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,9,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,9,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,9,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,9,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,9,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,9,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,9,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,9,5);
}
void fenetreNote0_10 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_10"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_10 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_10");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_10");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_10");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_10");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_10");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,10,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,10,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,10,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,10,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,10,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,10,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,10,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,10,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,10,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,10,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,10,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,10,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,10,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,10,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,10,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,10,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,10,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,10,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,10,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,10,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,10,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,10,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,10,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,10,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,10,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,10,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,10,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,10,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,10,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,10,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,10,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,10,5);
}

void fenetreNote0_11 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_11"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_11 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_11");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_11");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_11");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_11");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_11");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,11,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,11,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,11,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,11,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,11,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,11,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,11,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,11,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,11,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,11,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,11,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,11,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,11,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,11,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,11,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,11,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,11,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,11,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,11,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,11,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,11,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,11,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,11,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,11,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,11,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,11,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,11,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,11,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,11,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,11,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,11,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,11,5);
}

void fenetreNote0_12 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_12"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_12 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_12");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_12");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_12");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_12");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_12");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,12,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,12,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,12,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,12,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,12,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,12,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,12,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,12,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,12,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,12,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,12,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,12,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,12,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,12,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,12,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,12,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,12,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,12,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,12,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,12,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,12,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,12,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,12,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,12,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,12,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,12,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,12,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,12,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,12,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,12,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,12,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,12,5);
}

void fenetreNote0_13 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_13"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_13 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_13");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_13");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_13");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_13");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_13");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,13,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,13,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,13,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,13,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,13,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,13,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,13,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,13,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,13,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,13,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,13,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,13,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,13,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,13,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,13,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,13,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,13,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,13,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,13,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,13,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,13,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,13,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,13,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,13,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,13,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,13,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,13,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,13,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,13,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,13,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,13,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,13,5);
}

void fenetreNote0_14 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_14"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_14 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_14");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_14");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_14");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_14");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_14");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,14,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,14,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,14,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,14,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,14,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,14,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,14,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,14,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,14,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,14,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,14,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,14,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,14,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,14,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,14,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,14,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,14,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,14,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,14,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,14,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,14,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,14,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,14,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,14,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,14,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,14,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,14,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,14,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,14,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,14,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,14,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,14,5);
}

void fenetreNote0_15 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote0_15"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote0_15 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote0_15");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave0_15");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument0_15");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume0_15");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet0_15");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,0,15,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,0,15,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,0,15,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,0,15,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,0,15,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,0,15,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,0,15,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,0,15,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,0,15,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,0,15,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,0,15,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,0,15,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,0,15,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,0,15,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,0,15,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,0,15,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,0,15,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,0,15,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,0,15,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,0,15,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,0,15,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,0,15,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,0,15,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,0,15,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,0,15,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,0,15,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,0,15,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,0,15,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,0,15,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,0,15,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,0,15,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,0,15,5);
}

void fenetreNote1_0 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_0"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_0(GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_0");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_0");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_0");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_0");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_0");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,0,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,0,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,0,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,0,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,0,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,0,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,0,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,0,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,0,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,0,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,0,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,0,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,0,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,0,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,0,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,0,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,0,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,0,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,0,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,0,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,0,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,0,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,0,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,0,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,0,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,0,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,0,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,0,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,0,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,0,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,0,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,0,5);
}
void fenetreNote1_1 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_1"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_1 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_1");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_1");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_1");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_1");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_1");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,1,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,1,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,1,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,1,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,1,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,1,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,1,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,1,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,1,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,1,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,1,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,1,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,1,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,1,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,1,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,1,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,1,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,1,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,1,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,1,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,1,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,1,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,1,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,1,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,1,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,1,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,1,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,1,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,1,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,1,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,1,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,1,5);
}
void fenetreNote1_2 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_2"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_2 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_2");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_2");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_2");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_2");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_2");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,2,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,2,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,2,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,2,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,2,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,2,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,2,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,2,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,2,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,2,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,2,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,2,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,2,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,2,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,2,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,2,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,2,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,2,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,2,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,2,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,2,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,2,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,2,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,2,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,2,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,2,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,2,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,2,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,2,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,2,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,2,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,2,5);
}
void fenetreNote1_3 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_3"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_3 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_3");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_3");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_3");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_3");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_3");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,3,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,3,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,3,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,3,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,3,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,3,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,3,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,3,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,3,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,3,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,3,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,3,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,3,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,3,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,3,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,3,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,3,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,3,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,3,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,3,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,3,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,3,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,3,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,3,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,3,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,3,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,3,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,3,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,3,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,3,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,3,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,3,5);
}
void fenetreNote1_4 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_4"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_4 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_4");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_4");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_4");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_4");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_4");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,4,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,4,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,4,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,4,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,4,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,4,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,4,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,4,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,4,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,4,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,4,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,4,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,4,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,4,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,4,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,4,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,4,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,4,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,4,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,4,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,4,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,4,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,4,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,4,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,4,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,4,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,4,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,4,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,4,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,4,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,4,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,4,5);
}
void fenetreNote1_5 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_5"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_5 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_5");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_5");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_5");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_5");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_5");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,5,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,5,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,5,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,5,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,5,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,5,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,5,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,5,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,5,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,5,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,5,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,5,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,5,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,5,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,5,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,5,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,5,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,5,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,5,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,5,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,5,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,5,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,5,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,5,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,5,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,5,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,5,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,5,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,5,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,5,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,5,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,5,5);
}
void fenetreNote1_6 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_6"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_6 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_6");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_6");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_6");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_6");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_6");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,6,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,6,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,6,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,6,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,6,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,6,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,6,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,6,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,6,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,6,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,6,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,6,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,6,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,6,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,6,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,6,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,6,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,6,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,6,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,6,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,6,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,6,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,6,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,6,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,6,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,6,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,6,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,6,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,6,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,6,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,6,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,6,5);
}
void fenetreNote1_7 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_7"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_7 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_7");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_7");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_7");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_7");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_7");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,7,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,7,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,7,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,7,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,7,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,7,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,7,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,7,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,7,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,7,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,7,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,7,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,7,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,7,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,7,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,7,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,7,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,7,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,7,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,7,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,7,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,7,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,7,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,7,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,7,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,7,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,7,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,7,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,7,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,7,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,7,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,7,5);
}
void fenetreNote1_8 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_8"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_8 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_8");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_8");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_8");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_8");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_8");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,8,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,8,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,8,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,8,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,8,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,8,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,8,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,8,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,8,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,8,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,8,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,8,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,8,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,8,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,8,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,8,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,8,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,8,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,8,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,8,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,8,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,8,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,8,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,8,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,8,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,8,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,8,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,8,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,8,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,8,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,8,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,8,5);
}
void fenetreNote1_9 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_9"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_9 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_9");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_9");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_9");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_9");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_9");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,9,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,9,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,9,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,9,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,9,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,9,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,9,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,9,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,9,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,9,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,9,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,9,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,9,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,9,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,9,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,9,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,9,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,9,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,9,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,9,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,9,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,9,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,9,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,9,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,9,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,9,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,9,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,9,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,9,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,9,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,9,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,9,5);
}
void fenetreNote1_10 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_10"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_10 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_10");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_10");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_10");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_10");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_10");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,10,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,10,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,10,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,10,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,10,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,10,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,10,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,10,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,10,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,10,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,10,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,10,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,10,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,10,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,10,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,10,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,10,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,10,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,10,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,10,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,10,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,10,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,10,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,10,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,10,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,10,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,10,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,10,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,10,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,10,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,10,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,10,5);
}

void fenetreNote1_11 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_11"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_11 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_11");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_11");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_11");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_11");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_11");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,11,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,11,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,11,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,11,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,11,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,11,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,11,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,11,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,11,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,11,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,11,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,11,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,11,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,11,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,11,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,11,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,11,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,11,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,11,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,11,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,11,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,11,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,11,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,11,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,11,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,11,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,11,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,11,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,11,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,11,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,11,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,11,5);
}

void fenetreNote1_12 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_12"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_12 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_12");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_12");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_12");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_12");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_12");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,12,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,12,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,12,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,12,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,12,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,12,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,12,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,12,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,12,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,12,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,12,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,12,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,12,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,12,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,12,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,12,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,12,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,12,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,12,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,12,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,12,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,12,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,12,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,12,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,12,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,12,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,12,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,12,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,12,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,12,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,12,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,12,5);
}

void fenetreNote1_13 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_13"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_13 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_13");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_13");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_13");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_13");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_13");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,13,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,13,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,13,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,13,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,13,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,13,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,13,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,13,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,13,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,13,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,13,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,13,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,13,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,13,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,13,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,13,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,13,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,13,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,13,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,13,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,13,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,13,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,13,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,13,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,13,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,13,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,13,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,13,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,13,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,13,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,13,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,13,5);
}

void fenetreNote1_14 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_14"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_14 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_14");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_14");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_14");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_14");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_14");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,14,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,14,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,14,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,14,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,14,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,14,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,14,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,14,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,14,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,14,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,14,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,14,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,14,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,14,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,14,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,14,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,14,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,14,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,14,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,14,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,14,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,14,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,14,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,14,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,14,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,14,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,14,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,14,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,14,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,14,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,14,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,14,5);
}

void fenetreNote1_15 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote1_15"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote1_15 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote1_15");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave1_15");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument1_15");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume1_15");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet1_15");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,1,15,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,1,15,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,1,15,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,1,15,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,1,15,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,1,15,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,1,15,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,1,15,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,1,15,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,1,15,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,1,15,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,1,15,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,1,15,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,1,15,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,1,15,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,1,15,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,1,15,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,1,15,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,1,15,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,1,15,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,1,15,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,1,15,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,1,15,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,1,15,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,1,15,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,1,15,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,1,15,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,1,15,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,1,15,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,1,15,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,1,15,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,1,15,5);
}

void fenetreNote2_0 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_0"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_0 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_0");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_0");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_0");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_0");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_0");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,0,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,0,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,0,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,0,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,0,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,0,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,0,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,0,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,0,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,0,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,0,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,0,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,0,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,0,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,0,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,0,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,0,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,0,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,0,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,0,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,0,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,0,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,0,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,0,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,0,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,0,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,0,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,0,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,0,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,0,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,0,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,0,5);
}
void fenetreNote2_1 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_1"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_1 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_1");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_1");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_1");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_1");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_1");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,1,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,1,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,1,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,1,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,1,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,1,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,1,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,1,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,1,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,1,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,1,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,1,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,1,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,1,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,1,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,1,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,1,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,1,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,1,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,1,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,1,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,1,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,1,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,1,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,1,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,1,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,1,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,1,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,1,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,1,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,1,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,1,5);
}
void fenetreNote2_2 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_2"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_2 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_2");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_2");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_2");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_2");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_2");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,2,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,2,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,2,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,2,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,2,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,2,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,2,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,2,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,2,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,2,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,2,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,2,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,2,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,2,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,2,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,2,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,2,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,2,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,2,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,2,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,2,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,2,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,2,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,2,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,2,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,2,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,2,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,2,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,2,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,2,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,2,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,2,5);
}
void fenetreNote2_3 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_3"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_3 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_3");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_3");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_3");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_3");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_3");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,3,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,3,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,3,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,3,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,3,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,3,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,3,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,3,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,3,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,3,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,3,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,3,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,3,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,3,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,3,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,3,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,3,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,3,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,3,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,3,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,3,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,3,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,3,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,3,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,3,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,3,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,3,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,3,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,3,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,3,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,3,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,3,5);
}
void fenetreNote2_4 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_4"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_4 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_4");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_4");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_4");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_4");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_4");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,4,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,4,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,4,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,4,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,4,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,4,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,4,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,4,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,4,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,4,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,4,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,4,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,4,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,4,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,4,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,4,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,4,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,4,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,4,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,4,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,4,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,4,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,4,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,4,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,4,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,4,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,4,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,4,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,4,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,4,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,4,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,4,5);
}
void fenetreNote2_5 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_5"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_5 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_5");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_5");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_5");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_5");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_5");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,5,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,5,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,5,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,5,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,5,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,5,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,5,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,5,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,5,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,5,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,5,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,5,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,5,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,5,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,5,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,5,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,5,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,5,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,5,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,5,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,5,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,5,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,5,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,5,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,5,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,5,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,5,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,5,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,5,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,5,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,5,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,5,5);
}
void fenetreNote2_6 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_6"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_6 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_6");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_6");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_6");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_6");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_6");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,6,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,6,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,6,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,6,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,6,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,6,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,6,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,6,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,6,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,6,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,6,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,6,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,6,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,6,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,6,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,6,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,6,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,6,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,6,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,6,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,6,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,6,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,6,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,6,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,6,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,6,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,6,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,6,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,6,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,6,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,6,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,6,5);
}
void fenetreNote2_7 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_7"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_7 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_7");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_7");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_7");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_7");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_7");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,7,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,7,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,7,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,7,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,7,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,7,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,7,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,7,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,7,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,7,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,7,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,7,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,7,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,7,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,7,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,7,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,7,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,7,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,7,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,7,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,7,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,7,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,7,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,7,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,7,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,7,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,7,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,7,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,7,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,7,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,7,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,7,5);
}
void fenetreNote2_8 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_8"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_8 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_8");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_8");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_8");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_8");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_8");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,8,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,8,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,8,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,8,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,8,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,8,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,8,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,8,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,8,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,8,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,8,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,8,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,8,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,8,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,8,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,8,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,8,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,8,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,8,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,8,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,8,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,8,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,8,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,8,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,8,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,8,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,8,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,8,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,8,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,8,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,8,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,8,5);
}
void fenetreNote2_9 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_9"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_9 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_9");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_9");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_9");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_9");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_9");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,9,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,9,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,9,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,9,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,9,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,9,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,9,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,9,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,9,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,9,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,9,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,9,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,9,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,9,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,9,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,9,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,9,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,9,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,9,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,9,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,9,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,9,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,9,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,9,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,9,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,9,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,9,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,9,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,9,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,9,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,9,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,9,5);
}
void fenetreNote2_10 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_10"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_10 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_10");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_10");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_10");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_10");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_10");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,10,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,10,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,10,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,10,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,10,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,10,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,10,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,10,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,10,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,10,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,10,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,10,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,10,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,10,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,10,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,10,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,10,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,10,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,10,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,10,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,10,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,10,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,10,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,10,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,10,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,10,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,10,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,10,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,10,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,10,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,10,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,10,5);
}

void fenetreNote2_11 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_11"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_11 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_11");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_11");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_11");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_11");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_11");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,11,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,11,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,11,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,11,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,11,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,11,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,11,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,11,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,11,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,11,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,11,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,11,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,11,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,11,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,11,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,11,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,11,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,11,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,11,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,11,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,11,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,11,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,11,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,11,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,11,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,11,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,11,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,11,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,11,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,11,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,11,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,11,5);
}

void fenetreNote2_12 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_12"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_12 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_12");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_12");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_12");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_12");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_12");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,12,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,12,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,12,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,12,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,12,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,12,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,12,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,12,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,12,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,12,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,12,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,12,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,12,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,12,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,12,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,12,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,12,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,12,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,12,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,12,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,12,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,12,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,12,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,12,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,12,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,12,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,12,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,12,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,12,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,12,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,12,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,12,5);
}

void fenetreNote2_13 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_13"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_13 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_13");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_13");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_13");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_13");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_13");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,13,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,13,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,13,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,13,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,13,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,13,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,13,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,13,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,13,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,13,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,13,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,13,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,13,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,13,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,13,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,13,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,13,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,13,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,13,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,13,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,13,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,13,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,13,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,13,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,13,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,13,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,13,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,13,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,13,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,13,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,13,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,13,5);
}

void fenetreNote2_14 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_14"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_14 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_14");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_14");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_14");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_14");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_14");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,14,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,14,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,14,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,14,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,14,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,14,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,14,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,14,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,14,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,14,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,14,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,14,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,14,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,14,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,14,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,14,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,14,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,14,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,14,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,14,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,14,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,14,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,14,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,14,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,14,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,14,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,14,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,14,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,14,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,14,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,14,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,14,5);
}

void fenetreNote2_15 ( int argc, char *argv[] )
{
  GtkWidget *maFenetreNote = GTK_WIDGET( gtk_builder_get_object( monBuilder, "ajoutnote2_15"));
  
  gtk_widget_show_all (maFenetreNote);
}

void remplissageNote2_15 (GtkWidget *widget, gpointer *data){
Motif* m = melodie[0];

  GtkEntry *entryNote = (GtkEntry*) gtk_builder_get_object(monBuilder,"combonote2_15");
  GtkEntry *entryOctave = (GtkEntry*) gtk_builder_get_object(monBuilder,"combooctave2_15");
  GtkEntry *entryInstrument = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboinstrument2_15");
  GtkEntry *entryVolume = (GtkEntry*) gtk_builder_get_object(monBuilder,"combovolume2_15");
  GtkEntry *entryEffet = (GtkEntry*) gtk_builder_get_object(monBuilder,"comboeffet2_15");

  const char *strnote = gtk_entry_get_text(GTK_ENTRY(entryNote));
  const char *stroctave = gtk_entry_get_text(GTK_ENTRY(entryOctave));
  const char *strinstrument = gtk_entry_get_text(GTK_ENTRY(entryInstrument));
  const char *strvolume = gtk_entry_get_text(GTK_ENTRY(entryVolume));
  const char *streffet = gtk_entry_get_text(GTK_ENTRY(entryEffet));

  if (strcmp(Do,strnote)==0) definirNote(m,2,15,1);
  else 
    if (strcmp(DoD,strnote)==0) definirNote(m,2,15,2);
    else
      if (strcmp(Re,strnote)==0) definirNote(m,2,15,3);
      else
	if (strcmp(ReD,strnote)==0) definirNote(m,2,15,4);
	else
	  if (strcmp(Mi,strnote)==0) definirNote(m,2,15,5);
	  else
	    if (strcmp(Fa,strnote)==0) definirNote(m,2,15,6);
	    else
	      if (strcmp(FaD,strnote)==0) definirNote(m,2,15,7);
	      else
		if (strcmp(Sol,strnote)==0) definirNote(m,2,15,8);
		else
		  if (strcmp(SolD,strnote)==0) definirNote(m,2,15,9);
		  else
		    if (strcmp(La,strnote)==0) definirNote(m,2,15,10);
		    else
		      if (strcmp(LaD,strnote)==0) definirNote(m,2,15,11);
		      else
			if (strcmp(Si,strnote)==0) definirNote(m,2,15,12);

  if (strcmp(N1,stroctave)==0) definirOctave(m,2,15,1);
  else
    if (strcmp(N2,stroctave)==0) definirOctave(m,2,15,2);
    else
      if (strcmp(N3,stroctave)==0) definirOctave(m,2,15,3);
      else
	if (strcmp(N4,stroctave)==0) definirOctave(m,2,15,4);
	else
	  if (strcmp(N5,stroctave)==0) definirOctave(m,2,15,5);

  if (strcmp(N1,strinstrument)==0) definirInstrument(m,2,15,1);
  else
    if (strcmp(N2,strinstrument)==0) definirInstrument(m,2,15,2);
    else
      if (strcmp(N3,strinstrument)==0) definirInstrument(m,2,15,3);
      else
	if (strcmp(N4,strinstrument)==0) definirInstrument(m,2,15,4);
	else
	  if (strcmp(N5,strinstrument)==0) definirInstrument(m,2,15,5);

  if (strcmp(N1,strvolume)==0) definirVolume(m,2,15,1);
  else
    if (strcmp(N2,strvolume)==0) definirVolume(m,2,15,2);
    else
      if (strcmp(N3,strvolume)==0) definirVolume(m,2,15,3);
      else
	if (strcmp(N4,strvolume)==0) definirVolume(m,2,15,4);
	else
	  if (strcmp(N5,strvolume)==0) definirVolume(m,2,15,5);

  if (strcmp(N1,streffet)==0) definirEffet(m,2,15,1);
  else
    if (strcmp(N2,streffet)==0) definirEffet(m,2,15,2);
    else
      if (strcmp(N3,streffet)==0) definirEffet(m,2,15,3);
      else
	if (strcmp(N4,streffet)==0) definirEffet(m,2,15,4);
	else
	  if (strcmp(N5,streffet)==0) definirEffet(m,2,15,5);
}

void lancerLecture ()
{
	
	//courant = 0;
	//taille = 0;
	printf("%d\n",nbrPortees);
	//debut(melodie, inst, smpl, liste, nbrPortees);
	lecture(nbrPortees,1,480,inst,melodie);
}

void quitter ()
{
	gtk_main_quit();
}

void refresh(){}
